@@ -1,23 +1,8 @@
 /*
+ * Embedded Linux library
+ * Copyright (C) 2011-2014  Intel Corporation
  *
- *  Embedded Linux library
- *
- *  Copyright (C) 2011-2014  Intel Corporation. All rights reserved.
- *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Lesser General Public
- *  License as published by the Free Software Foundation; either
- *  version 2.1 of the License, or (at your option) any later version.
- *
- *  This library is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Lesser General Public License for more details.
- *
- *  You should have received a copy of the GNU Lesser General Public
- *  License along with this library; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- *
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #ifdef HAVE_CONFIG_H
@@ -1583,25 +1568,50 @@ bool _dbus_object_tree_add_interface(struct _dbus_object_tree *tree,
 	return true;
 }
 
-void *_dbus_object_tree_get_interface_data(struct _dbus_object_tree *tree,
+static struct interface_instance *_dbus_object_tree_find_interface(
+						struct _dbus_object_tree *tree,
 						const char *path,
 						const char *interface)
 {
 	struct object_node *object;
-	struct interface_instance *instance;
 
 	object = l_hashmap_lookup(tree->objects, path);
 	if (!object)
 		return NULL;
 
-	instance = l_queue_find(object->instances, match_interface_instance,
-				(char *) interface);
+	return l_queue_find(object->instances, match_interface_instance,
+				interface);
+}
+
+void *_dbus_object_tree_get_interface_data(struct _dbus_object_tree *tree,
+						const char *path,
+						const char *interface)
+{
+	struct interface_instance *instance;
+
+	instance = _dbus_object_tree_find_interface(tree, path, interface);
 	if (!instance)
 		return NULL;
 
 	return instance->user_data;
 }
 
+bool _dbus_object_tree_set_interface_data(struct _dbus_object_tree *tree,
+						const char *path,
+						const char *interface,
+						void *user_data)
+{
+	struct interface_instance *instance;
+
+	instance = _dbus_object_tree_find_interface(tree, path, interface);
+	if (!instance)
+		return false;
+
+	instance->user_data = user_data;
+
+	return true;
+}
+
 static bool match_object_manager_path(const void *a, const void *b)
 {
 	const struct object_manager *manager = a;
@@ -1655,11 +1665,11 @@ bool _dbus_object_tree_remove_interface(struct _dbus_object_tree *tree,
 		if (interfaces_added_rec && l_queue_remove(
 						interfaces_added_rec->instances,
 						instance)) {
-			if (l_queue_isempty(interfaces_added_rec->instances))
+			if (l_queue_isempty(interfaces_added_rec->instances)) {
 				l_queue_remove(manager->announce_added,
 						interfaces_added_rec);
-
-			interface_add_record_free(interfaces_added_rec);
+				interface_add_record_free(interfaces_added_rec);
+			}
 
 			continue;
 		}
@@ -1709,16 +1719,29 @@ void _dbus_object_tree_introspect(struct _dbus_object_tree *tree,
 {
 	struct object_node *node;
 	struct child_node *child;
+	bool path_is_object = true;
 
 	node = l_hashmap_lookup(tree->objects, path);
-	if (!node)
+	if (!node) {
+		path_is_object = false;
 		node = _dbus_object_tree_lookup(tree, path);
+	}
 
 	l_string_append(buf, XML_HEAD);
 	l_string_append(buf, "<node>\n");
 
 	if (node) {
-		l_string_append(buf, static_introspectable);
+		/*
+		 * We emit org.freedesktop.DBus.Introspectable only in case the
+		 * object node corresponds to a registered object, i.e.
+		 * exposes anything other than:
+		 * - org.freedesktop.DBus.Introspectable
+		 * - org.freedesktop.DBus.Peer
+		 * - org.freedesktop.DBus.Properties
+		 */
+		if (path_is_object)
+			l_string_append(buf, static_introspectable);
+
 		l_queue_foreach(node->instances,
 					generate_interface_instance, buf);
 
