@@ -12,6 +12,8 @@
 #include <config.h>
 #endif
 
+#include <time.h>
+
 #include <ell/ell.h>
 
 #include "mesh/mesh-defs.h"
@@ -519,7 +521,7 @@ void friend_poll(struct mesh_net *net, uint16_t src, bool seq,
 
 	frnd->u.active.seq = seq;
 	frnd->u.active.last = !seq;
-	md = !!(l_queue_length(frnd->pkt_cache) > 1);
+	md = (l_queue_length(frnd->pkt_cache) > 1);
 
 	if (pkt->ctl) {
 		/* Make sure we don't change the bit-sense of MD,
