@@ -104,6 +104,7 @@ static const char *br_options[] = {
 	"InquiryScanWindow",
 	"LinkSupervisionTimeout",
 	"PageTimeout",
+	"IdleTimeout",
 	"MinSniffInterval",
 	"MaxSniffInterval",
 	NULL
@@ -120,8 +121,8 @@ static const char *le_options[] = {
 	"ScanWindowSuspend",
 	"ScanIntervalDiscovery",
 	"ScanWindowDiscovery",
-	"ScanIntervalAdvMonitoring",
-	"ScanWindowAdvMonitoring",
+	"ScanIntervalAdvMonitor",
+	"ScanWindowAdvMonitor",
 	"ScanIntervalConnect",
 	"ScanWindowConnect",
 	"MinConnectionInterval",
@@ -494,8 +495,8 @@ struct config_param {
 	const char * const val_name;
 	void * const val;
 	const size_t size;
-	const uint16_t min;
-	const uint16_t max;
+	const uint32_t min;
+	const uint32_t max;
 };
 
 static void parse_mode_config(GKeyFile *config, const char *group,
@@ -573,6 +574,11 @@ static void parse_br_config(GKeyFile *config)
 		  sizeof(btd_opts.defaults.br.max_sniff_interval),
 		  0x0001,
 		  0xFFFE},
+		{ "IdleTimeout",
+		  &btd_opts.defaults.br.idle_timeout,
+		  sizeof(btd_opts.defaults.br.idle_timeout),
+		  500,
+		  3600000},
 	};
 
 	if (btd_opts.mode == BT_MODE_LE)
@@ -1330,6 +1336,8 @@ static char *option_noplugin = NULL;
 static char *option_configfile = NULL;
 static gboolean option_compat = FALSE;
 static gboolean option_detach = TRUE;
+static gboolean option_experimental = FALSE;
+static gboolean option_testing = FALSE;
 static gboolean option_version = FALSE;
 
 static void free_options(void)
@@ -1420,9 +1428,9 @@ static GOptionEntry options[] = {
 			"Specify an explicit path to the config file", "FILE"},
 	{ "compat", 'C', 0, G_OPTION_ARG_NONE, &option_compat,
 				"Provide deprecated command line interfaces" },
-	{ "experimental", 'E', 0, G_OPTION_ARG_NONE, &btd_opts.experimental,
+	{ "experimental", 'E', 0, G_OPTION_ARG_NONE, &option_experimental,
 				"Enable experimental D-Bus interfaces" },
-	{ "testing", 'T', 0, G_OPTION_ARG_NONE, &btd_opts.testing,
+	{ "testing", 'T', 0, G_OPTION_ARG_NONE, &option_testing,
 				"Enable testing D-Bus interfaces" },
 	{ "kernel", 'K', G_OPTION_FLAG_OPTIONAL_ARG, G_OPTION_ARG_CALLBACK,
 				parse_kernel_experimental,
@@ -1464,6 +1472,9 @@ int main(int argc, char *argv[])
 		exit(0);
 	}
 
+	btd_opts.experimental = option_experimental;
+	btd_opts.testing = option_testing;
+
 	umask(0077);
 
 	btd_backtrace_init();
