@@ -206,7 +206,7 @@
 #define LC3_CONFIG_32_2_AC(_ac) \
 	LC3_CONFIG_32_AC(LC3_CONFIG_DURATION_10, LC3_CONFIG_FRAME_LEN_32_2, _ac)
 
-#define LC3_CONFIG_FRAME_LEN_44_1 98u
+#define LC3_CONFIG_FRAME_LEN_44_1 97u
 
 #define LC3_CONFIG_44_1 \
 	LC3_CONFIG_44(LC3_CONFIG_DURATION_7_5, LC3_CONFIG_FRAME_LEN_44_1)
@@ -491,7 +491,7 @@
 	.ucast.io_qos.interval = _interval, \
 	.ucast.io_qos.latency = _lat, \
 	.ucast.io_qos.sdu = _sdu, \
-	.ucast.io_qos.phy = BT_BAP_CONFIG_PHY_2M, \
+	.ucast.io_qos.phys = BT_BAP_CONFIG_PHY_2M, \
 	.ucast.io_qos.rtn = _rtn, \
 }
 
@@ -947,7 +947,7 @@
 	.bcast.io_qos.interval = _interval, \
 	.bcast.io_qos.latency = _lat, \
 	.bcast.io_qos.sdu = _sdu, \
-	.bcast.io_qos.phy = BT_BAP_CONFIG_PHY_2M, \
+	.bcast.io_qos.phys = BT_BAP_CONFIG_PHY_2M, \
 	.bcast.io_qos.rtn = _rtn, \
 }
 
